@@ -4,7 +4,7 @@ class Config(PluginConfig):
     name = 'netbox_ping'
     verbose_name = 'NetBox Ping'
     description = 'Ping IPs and subnets'
-    version = '0.27'
+    version = '0.50'
     author = 'Christian Rose'
     default_settings = {
         'coming_soon': True
@@ -18,12 +18,6 @@ class Config(PluginConfig):
 
     # API settings
     base_url = 'netbox-ping'
-
-    # Register API serializers
-    def ready(self):
-        from .api.serializers import PluginSettingsModelSerializer
-        self.serializer_classes = {
-            'PluginSettingsModel': PluginSettingsModelSerializer,
-        }
+    default_app_config = 'netbox_ping.apps.NetBoxPingConfig'
 
 config = Config
\ No newline at end of file
