@@ -10,13 +10,16 @@ import subprocess
 import concurrent.futures
 from datetime import date
 from django.http import JsonResponse
+from django.utils import timezone
 
-from .utils import UnifiedInterface, natural_keys
+from .utils import UnifiedInterface, natural_keys, perform_dns_lookup
 from .forms import InterfaceComparisonForm
 from extras.models import Tag
 from .models import PluginSettingsModel
+import logging
 
 config = settings.PLUGINS_CONFIG['netbox_ping']
+logger = logging.getLogger('netbox.netbox_ping')
 
 
 class InterfaceComparisonView(LoginRequiredMixin, PermissionRequiredMixin, View):
@@ -171,6 +174,7 @@ class PingHomeView(LoginRequiredMixin, PermissionRequiredMixin, View):
             'model': Prefix,
             'table': None,
             'actions': ('add', 'import', 'export', 'bulk_edit', 'bulk_delete'),
+            'settings': settings,
             'update_tags': settings.update_tags,
         })
 
@@ -180,58 +184,50 @@ class PingHomeView(LoginRequiredMixin, PermissionRequiredMixin, View):
         settings.save()
         return JsonResponse({'status': 'success'})
 
+def get_timestamp():
+    """Helper function to get current timestamp as formatted string"""
+    return timezone.now().strftime('%Y-%m-%d %H:%M:%S')
+
 class PingSubnetView(LoginRequiredMixin, PermissionRequiredMixin, View):
     """View for pinging existing IPs in a subnet"""
     permission_required = "ipam.view_prefix"
 
     def ping_ip(self, ip):
-        """Ping a single IP address"""
+        """Ping an IP address and return tuple of (ip_str, is_alive)"""
         try:
-            # Try multiple pings to be more accurate
-            for _ in range(2):  # Try twice
-                result = subprocess.run(['ping', '-c', '1', '-W', '1', str(ip)], 
-                                    capture_output=True, 
-                                    timeout=2)
-                if result.returncode == 0:
-                    return str(ip), True
-            return str(ip), False
-        except:
+            subprocess.run(['ping', '-c', '1', '-W', '1', str(ip)], 
+                         stdout=subprocess.DEVNULL, 
+                         stderr=subprocess.DEVNULL, 
+                         check=True)
+            return str(ip), True
+        except subprocess.CalledProcessError:
             return str(ip), False
 
     def get(self, request, prefix_id):
-        # Get update_tags parameter
-        update_tags = request.GET.get('update_tags', 'true').lower() == 'true'
+        prefix = get_object_or_404(Prefix.objects.filter(id=prefix_id))
+        settings = PluginSettingsModel.get_settings()
+        update_tags = settings.update_tags
+        
+        # Get tags
+        online_tag = Tag.objects.get(name='online')
+        offline_tag = Tag.objects.get(name='offline')
         
-        prefix = get_object_or_404(Prefix, id=prefix_id)
         messages.info(request, f"🔍 Starting status check for subnet {prefix.prefix}")
-
-        try:
-            online_tag = Tag.objects.get(slug='online')
-            offline_tag = Tag.objects.get(slug='offline')
-        except Tag.DoesNotExist:
-            messages.error(request, "Required tags not found. Please initialize the plugin first.")
-            return redirect('plugins:netbox_ping:ping_home')
-
-        # Get all child IPs using NetBox's method
-        all_ips = prefix.get_child_ips()
         
-        if not all_ips.exists():
-            messages.warning(request, "No child IPs found in this subnet")
-            return redirect('ipam:prefix', pk=prefix_id)
-
+        # Track processed IPs and status changes
+        processed_ips = set()
         status_changes = []
-        processed_ips = set()  # Keep track of processed IPs
-
-        # Ping all IPs
+        
+        # Get all IPs in the prefix
+        ip_addresses = prefix.get_child_ips()
+        
+        # Ping all IPs in parallel
         with concurrent.futures.ThreadPoolExecutor(max_workers=20) as executor:
-            future_to_ip = {
-                executor.submit(self.ping_ip, str(ip_interface(ip.address).ip)): ip 
-                for ip in all_ips
-            }
+            future_to_ip = {executor.submit(self.ping_and_lookup_ip, ip.address.ip): ip for ip in ip_addresses}
             
             for future in concurrent.futures.as_completed(future_to_ip):
                 ip_obj = future_to_ip[future]
-                ip_str, is_alive = future.result()
+                ip_str, is_alive, hostname = future.result()
                 
                 # Skip if we've already processed this IP
                 if ip_str in processed_ips:
@@ -241,16 +237,35 @@ class PingSubnetView(LoginRequiredMixin, PermissionRequiredMixin, View):
                 # Initialize custom_field_data if needed
                 if not ip_obj.custom_field_data:
                     ip_obj.custom_field_data = {}
+                if 'Last_Seen' not in ip_obj.custom_field_data:
+                    ip_obj.custom_field_data['Last_Seen'] = None
                 
                 old_status = ip_obj.custom_field_data.get('Up_Down', None)
                 ip_obj.custom_field_data['Up_Down'] = is_alive
                 
+                # Update DNS name if found
+                if hostname:
+                    old_hostname = ip_obj.dns_name or ''  # Handle None case
+                    ip_obj.dns_name = hostname.lower()  # Ensure lowercase
+                    if old_hostname.lower() != hostname.lower():
+                        logger.info(f"Updated DNS name for {ip_str}: {hostname}")
+                else:
+                    # Set empty string instead of None
+                    ip_obj.dns_name = ''
+                
                 if is_alive:
+                    # Update Up_Down and Last_Seen status for successful pings
+                    ip_obj.custom_field_data['Up_Down'] = True
+                    ip_obj.custom_field_data['Last_Seen'] = get_timestamp()
                     if update_tags:
                         ip_obj.tags.remove(offline_tag)
                         ip_obj.tags.add(online_tag)
                     status = "🟢 up"
+                    if hostname:
+                        status += f" ({hostname})"
                 else:
+                    # Only update Up_Down for failed pings
+                    ip_obj.custom_field_data['Up_Down'] = False
                     if update_tags:
                         ip_obj.tags.remove(online_tag)
                         ip_obj.tags.add(offline_tag)
@@ -272,6 +287,51 @@ class PingSubnetView(LoginRequiredMixin, PermissionRequiredMixin, View):
 
         return redirect('ipam:prefix', pk=prefix_id)
 
+    def ping_and_lookup_ip(self, ip):
+        """Ping IP and perform DNS lookup"""
+        settings = PluginSettingsModel.get_settings()
+        
+        # First ping the IP
+        ip_str, is_alive = self.ping_ip(ip)
+        dns_name = None
+        
+        try:
+            ip_obj = IPAddress.objects.get(address=str(ip))
+            
+            if not ip_obj.custom_field_data:
+                ip_obj.custom_field_data = {}
+            
+            if is_alive:
+                # Update Up_Down and Last_Seen status
+                ip_obj.custom_field_data['Up_Down'] = True
+                ip_obj.custom_field_data['Last_Seen'] = get_timestamp()
+                ip_obj.save()
+                
+                # Perform DNS lookup if enabled
+                if settings.perform_dns_lookup:
+                    dns_servers = [s for s in [settings.dns_server1, settings.dns_server2, settings.dns_server3] if s]
+                    if dns_servers:
+                        hostname, verified = perform_dns_lookup(str(ip), dns_servers)
+                        if hostname:
+                            dns_name = hostname
+                            ip_obj.dns_name = hostname.lower()
+                            ip_obj.save()
+            else:
+                # IP is offline - only update Up_Down
+                ip_obj.custom_field_data['Up_Down'] = False
+                ip_obj.save()
+        
+        except IPAddress.DoesNotExist:
+            # For non-existent IPs, just return ping results
+            if is_alive and settings.perform_dns_lookup:
+                dns_servers = [s for s in [settings.dns_server1, settings.dns_server2, settings.dns_server3] if s]
+                if dns_servers:
+                    hostname, verified = perform_dns_lookup(str(ip), dns_servers)
+                    if hostname:
+                        dns_name = hostname
+
+        return ip_str, is_alive, dns_name
+
 def get_existing_ip(ip_str, prefix_length):
     """Helper function to check if IP exists (case-insensitive)"""
     try:
@@ -324,6 +384,7 @@ class ScanSubnetView(LoginRequiredMixin, PermissionRequiredMixin, View):
                     if ip_obj:
                         # Update existing IP
                         ip_obj.custom_field_data['Up_Down'] = True
+                        ip_obj.custom_field_data['Last_Seen'] = get_timestamp()
                         ip_obj.tags.remove(offline_tag)
                         ip_obj.tags.add(online_tag)
                         ip_obj.save()
@@ -340,6 +401,7 @@ class ScanSubnetView(LoginRequiredMixin, PermissionRequiredMixin, View):
                                     'Auto_discovered': str(date.today())
                                 }
                             )
+                            ip_obj.custom_field_data['Last_Seen'] = get_timestamp()
                             ip_obj.tags.add(online_tag)
                             ip_obj.tags.add(auto_discovered_tag)
                             messages.success(request, f"Added new IP {ip_str}")
@@ -353,6 +415,81 @@ class ScanSubnetView(LoginRequiredMixin, PermissionRequiredMixin, View):
 
         return redirect('ipam:prefix', pk=prefix_id)
 
+    def ping_and_lookup_ip(self, ip):
+        """Ping IP and perform DNS lookup"""
+        settings = PluginSettingsModel.get_settings()
+        
+        # First ping the IP
+        ip_str, is_alive = self.ping_ip(ip)
+        
+        try:
+            # Try to get existing IP object
+            ip_obj = IPAddress.objects.get(address=str(ip))
+            
+            # Always try DNS lookup if IP is alive
+            if is_alive:
+                dns_servers = [
+                    settings.dns_server1,
+                    settings.dns_server2,
+                    settings.dns_server3
+                ]
+                dns_servers = [s for s in dns_servers if s]  # Remove empty entries
+                
+                hostname, verified = perform_dns_lookup(str(ip), dns_servers)
+                if hostname:
+                    dns_name = hostname
+                    # Update the built-in dns_name field with a valid string
+                    ip_obj.dns_name = hostname.lower()  # Ensure lowercase
+                else:
+                    # Set empty string instead of None for dns_name
+                    ip_obj.dns_name = ''
+                
+                # Update Up_Down and Last_Seen status
+                ip_obj.custom_field_data['Up_Down'] = True
+                ip_obj.custom_field_data['Last_Seen'] = get_timestamp()
+                ip_obj.save()
+                
+                # Add online tag
+                online_tag = Tag.objects.get(name='online')
+                ip_obj.tags.add(online_tag)
+                
+                # Remove offline tag if it exists
+                try:
+                    offline_tag = Tag.objects.get(name='offline')
+                    ip_obj.tags.remove(offline_tag)
+                except Tag.DoesNotExist:
+                    pass
+            else:
+                # IP is offline - only update Up_Down, don't update Last_Seen
+                ip_obj.custom_field_data['Up_Down'] = False
+                ip_obj.save()
+                
+                # Add offline tag
+                offline_tag = Tag.objects.get(name='offline')
+                ip_obj.tags.add(offline_tag)
+                
+                # Remove online tag if it exists
+                try:
+                    online_tag = Tag.objects.get(name='online')
+                    ip_obj.tags.remove(online_tag)
+                except Tag.DoesNotExist:
+                    pass
+        
+        except IPAddress.DoesNotExist:
+            # IP doesn't exist in NetBox yet - just return the ping/DNS results
+            if is_alive:
+                dns_servers = [
+                    settings.dns_server1,
+                    settings.dns_server2,
+                    settings.dns_server3
+                ]
+                dns_servers = [s for s in dns_servers if s]
+                hostname, verified = perform_dns_lookup(str(ip), dns_servers)
+                if hostname:
+                    dns_name = hostname
+        
+        return ip_str, is_alive, dns_name
+
 class InitializePluginView(LoginRequiredMixin, PermissionRequiredMixin, View):
     """View for manually initializing plugin custom fields and tags"""
     permission_required = "ipam.view_prefix"
@@ -428,6 +565,7 @@ class ScanAllView(LoginRequiredMixin, PermissionRequiredMixin, View):
                                 if ip_obj:
                                     # Update existing IP
                                     ip_obj.custom_field_data['Up_Down'] = True
+                                    ip_obj.custom_field_data['Last_Seen'] = get_timestamp()
                                     ip_obj.tags.remove(offline_tag)
                                     ip_obj.tags.add(online_tag)
                                     ip_obj.save()
@@ -441,6 +579,7 @@ class ScanAllView(LoginRequiredMixin, PermissionRequiredMixin, View):
                                             'Auto_discovered': str(date.today())
                                         }
                                     )
+                                    ip_obj.custom_field_data['Last_Seen'] = get_timestamp()
                                     ip_obj.tags.add(online_tag)
                                     ip_obj.tags.add(auto_discovered_tag)
                                     total_discovered += 1
@@ -462,5 +601,90 @@ class UpdateSettingsView(LoginRequiredMixin, PermissionRequiredMixin, View):
     def post(self, request):
         settings = PluginSettingsModel.get_settings()
         settings.update_tags = request.POST.get('update_tags') == 'true'
+        settings.dns_server1 = request.POST.get('dns_server1', '').strip()
+        settings.dns_server2 = request.POST.get('dns_server2', '').strip()
+        settings.dns_server3 = request.POST.get('dns_server3', '').strip()
+        settings.perform_dns_lookup = request.POST.get('perform_dns_lookup') == 'true'
         settings.save()
-        return JsonResponse({'status': 'success'})
\ No newline at end of file
+        
+        return JsonResponse({
+            'status': 'success',
+            'data': {
+                'dns_server1': settings.dns_server1,
+                'dns_server2': settings.dns_server2,
+                'dns_server3': settings.dns_server3,
+                'perform_dns_lookup': settings.perform_dns_lookup,
+                'update_tags': settings.update_tags
+            }
+        })
+
+class PingSingleIPView(LoginRequiredMixin, PermissionRequiredMixin, View):
+    """View for pinging a single IP address"""
+    permission_required = "ipam.view_ipaddress"
+
+    def get(self, request, ip_address):
+        try:
+            # Split IP and prefix length
+            ip, prefix_length = ip_address.split('/')
+            
+            try:
+                ip_obj = IPAddress.objects.get(address=f"{ip}/{prefix_length}")
+            except IPAddress.DoesNotExist:
+                messages.error(request, f"IP {ip}/{prefix_length} does not exist in NetBox")
+                return redirect('ipam:ipaddress_list')
+            
+            # Initialize custom_field_data if needed
+            if not ip_obj.custom_field_data:
+                ip_obj.custom_field_data = {}
+            
+            # Reuse existing ping and lookup logic
+            ping_view = PingSubnetView()
+            ip_str, is_alive, hostname = ping_view.ping_and_lookup_ip(ip_obj.address.ip)
+            
+            if is_alive:
+                # Update Up_Down and Last_Seen status for successful pings
+                ip_obj.custom_field_data['Up_Down'] = True
+                ip_obj.custom_field_data['Last_Seen'] = get_timestamp()
+                ip_obj.save()
+                
+                status = "🟢 up"
+                if hostname:
+                    status += f" ({hostname})"
+                messages.success(request, f"IP {ip_str}: {status}")
+            else:
+                # Only update Up_Down for failed pings
+                ip_obj.custom_field_data['Up_Down'] = False
+                ip_obj.save()
+                messages.warning(request, f"IP {ip_str}: 🔴 down")
+            
+            return redirect('ipam:ipaddress', pk=ip_obj.pk)
+            
+        except ValueError as e:
+            messages.error(request, f"Invalid IP address format: {ip_address}")
+            return redirect('ipam:ipaddress_list')
+        except Exception as e:
+            messages.error(request, f"Error pinging IP: {str(e)}")
+            return redirect('ipam:ipaddress_list')
+
+class ScanSinglePrefixView(LoginRequiredMixin, PermissionRequiredMixin, View):
+    """View for scanning a single prefix"""
+    permission_required = "ipam.view_prefix"
+
+    def get(self, request, prefix):
+        try:
+            # Get prefix object
+            prefix_obj = get_object_or_404(Prefix, prefix=prefix)
+            
+            # Determine which action to take based on query parameter
+            action = request.GET.get('action', 'ping')
+            
+            if action == 'ping':
+                # Use existing ping logic
+                return PingSubnetView().get(request, prefix_obj.pk)
+            else:
+                # Use existing scan logic
+                return ScanSubnetView().get(request, prefix_obj.pk)
+                
+        except Exception as e:
+            messages.error(request, f"Error scanning prefix: {str(e)}")
+            return redirect('ipam:prefix_list')
\ No newline at end of file
